import { compose, id, } from "./utilities";
export const LOOKAROUND_DEFAULT = "group";
export function highlightWith(c) {
    return text => text.replace(c.pattern, c.mark);
}
export function proofreadWith(c) {
    const transformers = c.rules.map(markMistakesAndVerifiedUsing(Object.assign({ lookaround: c.lookaround || LOOKAROUND_DEFAULT }, c)));
    return transformers.reduceRight(compose, id);
}
export function markMistakesAndVerifiedUsing(c) {
    return r => compose(markAll("verified", c.markVerified, c.lookaround)(r), markAll("mistakes", c.markMistake, c.lookaround)(r));
}
export function markAll(kind, mark, lookaround) {
    return r => {
        const before = r.before === null ? "" : r.before.source;
        const after = r.after === null ? "" : r.after.source;
        const [f_b, f_m, f_a] = (lookaround === "native"
            ? [lookbehind, id, lookahead]
            : (lookaround === "group"
                ? [group, group, group]
                : [id, id, id]));
        const whatToFind = new RegExp(kind === "mistakes"
            ? f_b(before) + f_m(r.match.source) + f_a(after)
            : f_b(before) + f_m(r.match.source.split(r.bad).join(r.good)) + f_a(after), "g");
        const whatToMark = kind === "mistakes" ? r.bad : r.good;
        return findAndMarkUsing(whatToFind, whatToMark, mark, r.info, lookaround === "group");
    };
}
function group(r) {
    return "(" + r + ")";
}
function lookahead(r) {
    return "(?=" + r + ")";
}
function lookbehind(r) {
    return "(?<=" + r + ")";
}
function findAndMarkUsing(whatToFind, whatToMark, mark, info, fakeLookaround) {
    function markIn(s) {
        return s.split(whatToMark).join(mark(info)(whatToMark));
    }
    return text => text.replace(whatToFind, (found, ...rest) => {
        if (fakeLookaround) {
            const captured = rest.slice(0, 3);
            return captured[0] + markIn(captured[1]) + captured[2];
        }
        else {
            return markIn(found);
        }
    });
}
//# sourceMappingURL=core.js.map